<?php
/**
 * class-groups-ws-admin-order.php
 *
 * Copyright (c) "kento" Karim Rahimpur www.itthinx.com
 *
 * This code is provided subject to the license granted.
 * Unauthorized use and distribution is prohibited.
 * See COPYRIGHT.txt and LICENSE.txt
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This header and all notices must be kept intact.
 *
 * @author Karim Rahimpur
 * @package groups-woocommerce
 * @since groups-woocommerce 1.32.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Admin extensions to the order post type.
 */
class Groups_WS_Admin_Order {

	const ANY = '#any#';

	const NONE = '#none#';

	const POST_TYPE = 'shop_order';

	/**
	 * Hooks.
	 */
	public static function init() {
		if ( is_admin() ) {
			add_filter( 'posts_where', array( __CLASS__, 'posts_where' ), 10, 2 );
			add_action( 'admin_head', array( __CLASS__, 'admin_head' ) );
			add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_enqueue_scripts' ) );
			add_action( 'restrict_manage_posts', array( __CLASS__, 'restrict_manage_posts' ), 100 );
			add_action( 'add_meta_boxes', array( __CLASS__, 'add_meta_boxes' ) );
		}
	}

	/**
	 * CSS for product groups filter field.
	 */
	public static function admin_head() {

		global $pagenow;

		if ( $pagenow == 'edit.php' ) {
			$post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : null;
			if ( $post_type === self::POST_TYPE ) {
				echo '<style type="text/css">';
				echo '.groups-woocommerce-groups-container { display: inline-block; line-height: 24px; padding-bottom: 1em; vertical-align: top; margin-left: 4px; margin-right: 4px; }';
				echo '.groups-woocommerce-groups-container .selectize-control { min-width: 128px; }';
				echo '.groups-woocommerce-groups-container .selectize-control, .groups-bulk-container select.groups-action { margin-right: 4px; vertical-align: top; }';
				echo '.groups-woocommerce-groups-container .selectize-input { min-height: 30px; font-size: inherit; line-height: 18px; padding: 0 4px; vertical-align: middle; }';
				echo '.groups-woocommerce-groups-container .selectize-input input[type="text"] { font-size: inherit; vertical-align: middle; }';
				echo '.groups-woocommerce-groups-container input.button { margin-top: 1px; vertical-align: top; }';
				echo '.tablenav .actions { overflow: visible; }'; // make sure that selectize options are visible
				echo '.wp-list-table td { overflow: visible; }'; // make sure that bulk actions are visible
				echo '</style>';
			}
		}
	}

	/**
	 * Enqueues necessary scripts and styles for order admin screens.
	 */
	public static function admin_enqueue_scripts() {
		global $pagenow;
		if ( $pagenow == 'edit.php' ) {
			$post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : null;
			if ( $post_type === self::POST_TYPE ) {
				Groups_UIE::enqueue( 'select' );
			}
		}
		if ( $pagenow === 'post.php' ) {
			$screen    = get_current_screen();
			$screen_id = $screen ? $screen->id : '';
			if ( $screen_id === self::POST_TYPE ) {
				wp_enqueue_style( 'groups-woocommerce-admin-order' );
			}
		}
	}

	/**
	 * Renders the group filter field to filter orders by products with group added or removed.
	 */
	public static function restrict_manage_posts() {

		global $pagenow;

		if ( is_admin() ) {
			if ( $pagenow == 'edit.php' ) {
				$post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : null;
				if ( $post_type === self::POST_TYPE ) {
					// Product Groups ...
					$output = '';
					$output .= '<div class="groups-woocommerce-groups-container">';
					$output .= sprintf(
						'<select class="groups-woocommerce-product-groups" name="%s[]" multiple="multiple" placeholder="%s" data-placeholder="%s">',
						esc_attr( 'groups-woocommerce-product-groups' ),
						esc_attr( __( 'Product Groups &hellip;', 'groups-woocommerce' ) ) ,
						esc_attr( __( 'Product Groups &hellip;', 'groups-woocommerce' ) )
					);

					$previous_selected = array();
					if ( !empty( $_GET['groups-woocommerce-product-groups'] ) ) {
						$previous_selected = $_GET['groups-woocommerce-product-groups'];
						if ( !is_array( $previous_selected ) ) {
							$previous_selected = array();
						}
						if ( in_array( self::ANY, $previous_selected ) ) {
							$previous_selected = array( self::ANY );
						} else if ( in_array( self::NONE, $previous_selected ) ) {
							$previous_selected = array( self::NONE );
						}
					}

					$output .= sprintf(
						'<option value="%s" %s >%s</option>', self::NONE,
						esc_attr( in_array( self::NONE, $previous_selected ) ? ' selected="selected" ' : '' ),
						esc_attr( __( '(none)', 'groups-woocommerce' ) )
					);
					$output .= sprintf(
						'<option value="%s" %s >%s</option>', self::ANY,
						esc_attr( in_array( self::ANY, $previous_selected ) ? ' selected="selected" ' : '' ),
						esc_attr( __( '(any)', 'groups-woocommerce' ) )
					);

					$groups = Groups_Group::get_groups( array( 'order_by' => 'name', 'order' => 'ASC' ) );
					foreach( $groups as $group ) {
						$selected = in_array( $group->group_id, $previous_selected ) ? ' selected="selected" ' : '';
						$output .= sprintf(
							'<option value="%s" %s >%s</option>',
							esc_attr( $group->group_id ),
							esc_attr( $selected ),
							stripslashes( wp_filter_nohtml_kses( $group->name ) )
						);
					}
					$output .= '</select>';
					$output .= '</div>';
					$output .= Groups_UIE::render_select( '.groups-woocommerce-product-groups' );

					echo $output;
				}
			}
		}
	}

	/**
	 * Render the list of groups related to a product - includes groups to which
	 * the user is added and those from which the user is removed.
	 *
	 * @param int $post_id Product's post ID
	 *
	 * @return string
	 */
	public static function render_group_list( $post_id ) {

		global $wpdb;

		wp_enqueue_style( 'groups-woocommerce-admin' );

		$output = '';

		$product_groups        = get_post_meta( $post_id, '_groups_groups', false );
		$product_groups_remove = get_post_meta( $post_id, '_groups_groups_remove', false );

		$group_table = _groups_get_tablename( 'group' );
		$groups = $wpdb->get_results( "SELECT * FROM $group_table ORDER BY name" );

		if ( count( $groups ) > 0 ) {
			$list = array();
			foreach( $groups as $group ) {
				if ( is_array( $product_groups ) && in_array( $group->group_id, $product_groups ) ) {
					$list[] = stripslashes( wp_filter_nohtml_kses( $group->name ) );
				}
				if ( is_array( $product_groups_remove ) && in_array( $group->group_id, $product_groups_remove ) ) {
					$list[] = '(' . stripslashes( wp_filter_nohtml_kses( $group->name ) ) . ')';
				}
			}
			$output .= '<div class="list-groups">';
			$output .= implode( ', ', $list );
			$output .= '</div>'; // .list-groups
		}

		// variations
		$product = wc_get_product( $post_id );
		if ( $product->is_type( 'variable' ) ) {
			$variation_ids = $product->get_children();
			foreach( $variation_ids as $variation_id ) {
				if ( $variation = wc_get_product( $variation_id ) ) {
					$variation_name = $variation->get_name();
					$variation_groups = get_post_meta( $variation_id, '_groups_variation_groups', false );
					$variation_groups_remove = get_post_meta( $variation_id, '_groups_variation_groups_remove', false );
					$list = array();
					if (
						is_array( $variation_groups ) && count( $variation_groups ) > 0 ||
						is_array( $variation_groups_remove ) && count( $variation_groups_remove ) > 0
					) {
						foreach ( $groups as $group ) {
							if ( in_array( $group->group_id, $variation_groups ) ) {
								$list[] = stripslashes( wp_filter_nohtml_kses( $group->name ) );
							}
							if ( in_array( $group->group_id, $variation_groups_remove ) ) {
								$list[] = '(' . stripslashes( wp_filter_nohtml_kses( $group->name ) ) . ')';
							}
						}
					}
					if ( count( $list ) > 0 ) {
						$output .= '<div class="variation">';
						$output .= '<span class="variation-name">';
						$output .= wp_filter_nohtml_kses( $variation_name );
						$output .= ':';
						$output .= '</span>';
						$output .= '<div class="list-groups">';
						$output .= implode( ', ', $list );
						$output .= '</div>'; // .list-groups
						$output .= '</div>'; // .variation
					}
				}
			}
		}
		return $output;
	}

	/**
	 * Filter order by products with group memberships added or removed via product or product variation.
	 *
	 * @param string $where
	 * @param WP_Query $query
	 *
	 * @return string
	 */
	public static function posts_where( $where, $query ) {
		global $wpdb;

		if ( !$query->is_main_query() ) {
			return $where;
		}

		if ( $query->query_vars['post_type'] !== self::POST_TYPE ) {
			return $where;
		}

		if ( !empty( $_GET['groups-woocommerce-product-groups'] ) && is_array( $_GET['groups-woocommerce-product-groups'] ) ) {
			if ( in_array( self::ANY, $_GET['groups-woocommerce-product-groups'] ) ) {
				$condition =
					"SELECT DISTINCT post_id FROM $wpdb->postmeta " .
					"WHERE meta_key IN ( '_groups_groups', '_groups_groups_remove', '_groups_variation_groups', '_groups_variation_groups_remove' ) ";
				$order_ids = array();
				$product_ids = array();
				$products = $wpdb->get_results( $condition );
				if ( is_array( $products ) ) {
					foreach ( $products as $product ) {
						$product_ids[] = (int) $product->post_id;
					}
				}
				if ( count( $product_ids ) > 0 ) {
					$orders = $wpdb->get_results(
						"SELECT DISTINCT order_items.order_id AS order_id FROM {$wpdb->prefix}woocommerce_order_items AS order_items " .
						"LEFT JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS order_item_meta ON order_items.order_item_id = order_item_meta.order_item_id " .
						"WHERE " .
						"order_items.order_item_type = 'line_item' " .
						"AND order_item_meta.meta_key IN ( '_product_id', '_variation_id' ) " .
						"AND order_item_meta.meta_value IN ( " . implode( ',', $product_ids ) . " )"
					);
					if ( is_array( $orders ) ) {
						foreach ( $orders as $order ) {
							$order_ids[] = (int) $order->order_id;
						}
					}
				}
				if ( count( $order_ids ) === 0 ) {
					$order_ids = array( -1 );
				}
				$where .= " AND ( $wpdb->posts.ID IN ( " . implode( ',', $order_ids ) . ") ) ";
			} else if ( in_array( self::NONE, $_GET['groups-woocommerce-product-groups'] ) ) {
				$condition =
					"SELECT DISTINCT post_id FROM $wpdb->postmeta " .
					"WHERE meta_key IN ( '_groups_groups', '_groups_groups_remove', '_groups_variation_groups', '_groups_variation_groups_remove' ) ";
				$order_ids = array();
				$product_ids = array();
				$products = $wpdb->get_results( $condition );
				if ( is_array( $products ) ) {
					foreach ( $products as $product ) {
						$product_ids[] = (int) $product->post_id;
					}
				}
				if ( count( $product_ids ) > 0 ) {
					$orders = $wpdb->get_results(
						"SELECT DISTINCT order_items.order_id AS order_id FROM {$wpdb->prefix}woocommerce_order_items AS order_items " .
						"LEFT JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS order_item_meta ON order_items.order_item_id = order_item_meta.order_item_id " .
						"WHERE " .
						"order_items.order_item_type = 'line_item' " .
						"AND order_item_meta.meta_key IN ( '_product_id', '_variation_id' ) " .
						"AND order_item_meta.meta_value IN ( " . implode( ',', $product_ids ) . " )"
					);
					if ( is_array( $orders ) ) {
						foreach ( $orders as $order ) {
							$order_ids[] = (int) $order->order_id;
						}
					}
				}
				if ( count( $order_ids ) > 0 ) {
					$where .= " AND ( $wpdb->posts.ID NOT IN ( " . implode( ',', $order_ids ) . ") ) ";
				}
			} else {
				$group_ids = array();
				foreach ( $_GET['groups-woocommerce-product-groups'] as $group_id ) {
					if ( $group_id = Groups_Utility::id( $group_id ) ) {
						if ( Groups_Group::read( $group_id ) ) {
							$group_ids[] = $group_id;
						}
					}
				}
				$group_ids = array_unique( $group_ids );
				if ( count( $group_ids ) > 0 ) {
					$groups = ' ( ' . implode( ',', esc_sql( $group_ids ) ) . ' ) ';
					$condition =
						"SELECT DISTINCT post_id FROM $wpdb->postmeta " .
						"WHERE meta_key IN ( '_groups_groups', '_groups_groups_remove', '_groups_variation_groups', '_groups_variation_groups_remove' ) AND meta_value IN $groups";
					$order_ids = array();
					$product_ids = array();
					$products = $wpdb->get_results( $condition );
					if ( is_array( $products ) ) {
						foreach ( $products as $product ) {
							$product_ids[] = (int) $product->post_id;
						}
					}
					if ( count( $product_ids ) > 0 ) {
						$orders = $wpdb->get_results(
							"SELECT DISTINCT order_items.order_id AS order_id FROM {$wpdb->prefix}woocommerce_order_items AS order_items " .
							"LEFT JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS order_item_meta ON order_items.order_item_id = order_item_meta.order_item_id " .
							"WHERE " .
							"order_items.order_item_type = 'line_item' " .
							"AND order_item_meta.meta_key IN ( '_product_id', '_variation_id' ) " .
							"AND order_item_meta.meta_value IN ( " . implode( ',', $product_ids ) . " )"
						);
						if ( is_array( $orders ) ) {
							foreach ( $orders as $order ) {
								$order_ids[] = (int) $order->order_id;
							}
						}
					}
					if ( count( $order_ids ) === 0 ) {
						$order_ids = array( -1 );
					}
					$where .= " AND ( $wpdb->posts.ID IN ( " . implode( ',', $order_ids ) . ") ) ";
				}
			}
		}
		return $where;
	}

	/**
	 * Add the group membership box.
	 */
	public static function add_meta_boxes() {
		$screen    = get_current_screen();
		$screen_id = $screen ? $screen->id : '';
		if ( $screen_id === self::POST_TYPE ) {
			add_meta_box(
				'groups-woocommerce-order-product-groups',
				__( 'Group Memberships', 'groups-woocommerce' ),
				array( __CLASS__, 'order_product_groups_meta_box' ),
				self::POST_TYPE,
				'side',
				'default'
			);
		}
	}

	/**
	 * Check for array equality.
	 *
	 * @param array $a
	 * @param array $b
	 *
	 * @return boolean
	 */
	private static function array_equal( $a, $b ) {
		if ( !is_array( $a ) ) {
			$a = array();
		}
		if ( !is_array( $b ) ) {
			$b = array();
		}
		$distinct =
			count( array_diff( $a, $b ) ) > 0 ||
			count( array_diff( $b, $a ) ) > 0;
		return !$distinct;
	}

	/**
	 * Renders the content of the group membership box.
	 *
	 * @param WP_Post $post
	 */
	public static function order_product_groups_meta_box( $post ) {

		if ( !isset( $post->ID ) ) {
			return;
		}
		if ( $post->post_type !== self::POST_TYPE ) {
			return;
		}
		$order_id = $post->ID;
		/**
		 * @var WC_Order $order
		 */
		$order = wc_get_order( $order_id );
		if ( !( $order instanceof WC_Order ) ) {
			return;
		}

		$user_id = $order->get_customer_id();

		$groups_product_groups = get_user_meta( $user_id, '_groups_product_groups', true );
		$groups_product_groups_remove = get_user_meta( $user_id, '_groups_product_groups_remove', true );

		$user_product_groups = null;
		if ( isset( $groups_product_groups[$order_id] ) ) {
			$user_product_groups = $groups_product_groups[$order_id];
		}
		$user_product_groups_remove = null;
		if ( isset( $groups_product_groups_remove[$order_id] ) ) {
			$user_product_groups_remove = $groups_product_groups_remove[$order_id];
		}

		//
		// Gather
		//
		$entries = array();

		/**
		 * @var WC_Order_Item[] $items
		 */
		$items = $order->get_items();
		foreach ( $items as $key => $item ) {
			/**
			 * @var WC_Product $product
			 */
			$product = $item->get_product();
			if ( is_object( $product ) ) {

				$product_id = null;
				if ( !$product->is_type( 'variation' ) ) {
					$product_id = $product->get_id();
				} else {
					$product_id = $product->get_parent_id();
				}

				$is_subscription = class_exists( 'WC_Subscriptions_Product' ) && WC_Subscriptions_Product::is_subscription( $product_id );

				if ( $product_id ) {

					$entry = array();
					$entry['product_id'] = $product_id;
					$entry['variation_id'] = $product->is_type( 'variation' ) ? $product->get_id() : null;
					$entry['name'] = $product->get_name();
					$entry['groups'] = array();
					$entry['groups_remove'] = array();
					$entry['duration'] = null;
					$entry['duration_uom'] = null;
					$entry['duration_name'] = null;

					$product_groups        = get_post_meta( $product_id, '_groups_groups', false );
					$product_groups_remove = get_post_meta( $product_id, '_groups_groups_remove', false );
					$duration              = get_post_meta( $product_id, '_groups_duration', true );
					$duration_uom          = get_post_meta( $product_id, '_groups_duration_uom', true );
					if ( empty( $duration ) ) {
						$duration = null;
					}
					if ( empty( $duration_uom ) ) {
						$duration_uom = null;
					}

					if ( $product->is_type( 'variation' ) ) {
						$variation_groups        = get_post_meta( $product->get_id(), '_groups_variation_groups', false );
						$variation_groups_remove = get_post_meta( $product->get_id(), '_groups_variation_groups_remove', false );
						if ( !empty( $variation_groups ) ) {
							$product_groups = array_unique( array_merge( $product_groups, $variation_groups ) );
						}
						if ( !empty( $variation_groups_remove ) ) {
							$product_groups_remove = array_unique( array_merge( $product_groups_remove, $variation_groups_remove ) );
						}
					}

					// Override the current product group data with stored group data for the user
					$_product_groups = null;
					$_product_groups_remove = null;
					$_duration = null;
					$_duration_uom = null;
					if ( $user_product_groups !== null ) {
						if (
							isset( $user_product_groups[$product->get_id()] ) &&
							isset( $user_product_groups[$product->get_id()]['groups'] ) &&
							is_array( $user_product_groups[$product->get_id()]['groups'] )
						) {
							$_product_groups = $user_product_groups[$product->get_id()]['groups'];
						}
						if ( !empty( $user_product_groups[$product->get_id()]['duration'] ) ) {
							$_duration = $user_product_groups[$product->get_id()]['duration'];
						}
						if ( !empty( $user_product_groups[$product->get_id()]['duration_uom'] ) ) {
							$_duration_uom = $user_product_groups[$product->get_id()]['duration_uom'];
						}
					}
					if ( $user_product_groups_remove !== null ) {
						if (
							isset( $user_product_groups_remove[$product->get_id()] ) &&
							isset( $user_product_groups_remove[$product->get_id()]['groups'] ) &&
							is_array( $user_product_groups_remove[$product->get_id()]['groups'] )
						) {
							$_product_groups_remove = $user_product_groups_remove[$product->get_id()]['groups'];
						}
					}

					$stored = false;
					if ( $_product_groups !== null ) {
						if ( !self::array_equal( $product_groups, $_product_groups ) ) {
							$product_groups = $_product_groups;
							$stored = true;
						}
					}
					if ( $_product_groups_remove !== null ) {
						if ( !self::array_equal( $product_groups_remove, $_product_groups_remove ) ) {
							$product_groups_remove = $_product_groups_remove;
							$stored = true;
						}
					}
					if ( $_duration !== null ) {
						if ( $duration != $_duration ) {
							$duration = $_duration;
							$stored = true;
						}
					}
					if ( $_duration_uom !== null ) {
						if ( $duration_uom != $_duration_uom ) {
							$duration_uom = $_duration_uom;
							$stored = true;
						}
					}
					$entry['stored'] = $stored;

					if ( !empty( $duration ) ) {
						$entry['duration'] = $duration;
						$entry['duration_uom'] = !empty( $duration_uom ) ? $duration_uom : null;
						$duration_name = '';
						switch ( $duration_uom ) {
							case 'second':
								$duration_name = _n( 'Second', 'Seconds', $duration, 'groups-woocommerce' );
								break;
							case 'minute':
								$duration_name = _n( 'Minute', 'Minutes', $duration, 'groups-woocommerce' );
								break;
							case 'hour':
								$duration_name = _n( 'Hour', 'Hours', $duration, 'groups-woocommerce' );
								break;
							case 'day':
								$duration_name = _n( 'Day', 'Days', $duration, 'groups-woocommerce' );
								break;
							case 'week':
								$duration_name = _n( 'Week', 'Weeks', $duration, 'groups-woocommerce' );
								break;
							case 'year':
								$duration_name = _n( 'Year', 'Years', $duration, 'groups-woocommerce' );
								break;
							default:
								$duration_name = _n( 'Month', 'Months', $duration, 'groups-woocommerce' );
						}
						$entry['duration_name'] = $duration_name;
					}

					if ( $is_subscription ) {
						$entry['duration'] = '';
						$entry['duration_name'] = __( 'Subscription', 'groups-woocommerce' );
					}

					foreach ( $product_groups as $group_id ) {
						$group = Groups_Group::read( $group_id );
						if ( $group ) {
							$name = stripslashes( wp_filter_nohtml_kses( $group->name ) );
							$entry['groups'][] = $name;
						}
					}

					foreach ( $product_groups_remove as $group_id ) {
						$group = Groups_Group::read( $group_id );
						if ( $group ) {
							$name = stripslashes( wp_filter_nohtml_kses( $group->name ) );
							$entry['groups_remove'][] = $name;
						}
					}

					if ( !empty( $entry['groups'] ) || !empty( $entry['groups_remove'] ) ) {
						$entries[$key] = $entry;
					}
				}
			}
		}

		//
		// Render
		//
		if ( count( $entries ) > 0 ) {
			echo '<div class="groups-woocommerce-order-product-groups">';
			echo '<ul class="order-product-entries">';
			foreach ( $entries as $item_id => $entry ) {
				echo '<li class="order-product-entry">';
				echo '<table>';
				echo '<tbody>';

				echo '<tr>';
				echo '<th>';
				echo esc_html__( 'Product', 'groups-woocommerce' );
				echo '</th>';
				echo '<td>';
				echo esc_html( $entry['name'] );
				echo '</td>';
				echo '</tr>';

				echo '<tr>';
				echo '<th>';
				echo esc_html__( 'Duration', 'groups-woocommerce' );
				echo '</th>';
				echo '<td>';
				if ( $entry['duration'] !== null ) {
					echo esc_html( trim( sprintf( '%s %s', $entry['duration'], $entry['duration_name'] !== null ? $entry['duration_name'] : '' ) ) );
				} else {
					echo esc_html__( 'Unlimited', 'groups-woocommerce' );
				}
				echo '</td>';
				echo '</tr>';

				if ( count( $entry['groups'] ) > 0 ) {
					echo '<tr>';
					echo '<th>';
					echo esc_html__( 'Add to Groups', 'groups-woocommerce' );
					echo '</th>';

					echo '<td>';
					echo implode( ', ', array_map( 'esc_html', $entry['groups'] ) );
					echo '</td>';
					echo '</tr>';
				}

				if ( count( $entry['groups_remove'] ) > 0 ) {
					echo '<tr>';
					echo '<th>';
					echo esc_html__( 'Remove from Groups', 'groups-woocommerce' );
					echo '</th>';

					echo '<td>';
					echo implode( ', ', array_map( 'esc_html', $entry['groups_remove'] ) );
					echo '<td>';
					echo '</tr>';
				}

				echo '</tbody>';
				echo '</table>';
				if ( $entry['stored'] ) {
					echo '<p>';
					printf(
						'<span style="cursor:help" title="%s">%s</span>',
						esc_attr__( "The stored group membership data of this customer for the product in this order differs from product's current group membership settings.", 'groups-woocommerce' ),
						esc_html__( 'Stored membership data.', 'groups-woocommerce' )
					);
					echo '</p>';
				}
				echo '</li>';
			}
			echo '</ul>';

			echo '</div>'; // .groups-woocommerce-order-product-groups
		} else {
			echo esc_html__( 'None', 'groups-woocommerce' );
		}
	}
}
Groups_WS_Admin_Order::init();
