<?php
/**
 * class-groups-ws-admin-product.php
 *
 * Copyright (c) "kento" Karim Rahimpur www.itthinx.com
 *
 * This code is provided subject to the license granted.
 * Unauthorized use and distribution is prohibited.
 * See COPYRIGHT.txt and LICENSE.txt
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This header and all notices must be kept intact.
 *
 * @author Karim Rahimpur
 * @package groups-woocommerce
 * @since groups-woocommerce 1.5.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Admin extensions to the product post type.
 */
class Groups_WS_Admin_Product {

	const ANY = '#any#';

	const NONE = '#none#';

	/**
	 * Hooks.
	 */
	public static function init() {
		if ( is_admin() ) { // yeah yeah ... just in case
			add_filter( 'manage_product_posts_columns', array( __CLASS__, 'manage_product_posts_columns' ) );
			add_action( 'manage_product_posts_custom_column', array( __CLASS__, 'manage_product_posts_custom_column' ), 10, 2 );
			add_filter( 'manage_edit-product_sortable_columns', array( __CLASS__, 'manage_edit_product_sortable_columns' ) );
			// Related method is deprecated in this class: add_action( 'admin_init', array( __CLASS__, 'admin_init' ) );

			add_filter( 'posts_where', array( __CLASS__, 'posts_where' ), 10, 2 );
			add_filter( 'posts_join', array( __CLASS__, 'posts_join' ), 10, 2 );
			add_filter( 'posts_orderby', array( __CLASS__, 'posts_orderby' ), 10, 2 );

			add_action( 'admin_head', array( __CLASS__, 'admin_head' ) );
			add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_enqueue_scripts' ) );
			add_action( 'restrict_manage_posts', array( __CLASS__, 'restrict_manage_posts' ), 100 );
		}
	}

	/**
	 * CSS for product groups filter field.
	 */
	public static function admin_head() {

		global $pagenow;

		if ( $pagenow == 'edit.php' ) {
			$post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : null;
			if ( $post_type === 'product' ) {
				echo '<style type="text/css">';
				echo '.groups-woocommerce-groups-container { display: inline-block; line-height: 24px; padding-bottom: 1em; vertical-align: top; margin-left: 4px; margin-right: 4px; }';
				echo '.groups-woocommerce-groups-container .selectize-control { min-width: 128px; }';
				echo '.groups-woocommerce-groups-container .selectize-control, .groups-bulk-container select.groups-action { margin-right: 4px; vertical-align: top; }';
				echo '.groups-woocommerce-groups-container .selectize-input { min-height: 30px; font-size: inherit; line-height: 18px; padding: 0 4px; vertical-align: middle; }';
				echo '.groups-woocommerce-groups-container .selectize-input input[type="text"] { font-size: inherit; vertical-align: middle; }';
				echo '.groups-woocommerce-groups-container input.button { margin-top: 1px; vertical-align: top; }';
				echo '.tablenav .actions { overflow: visible; }'; // make sure that selectize options are visible
				echo '.wp-list-table td { overflow: visible; }'; // make sure that bulk actions are visible
				echo '</style>';
			}
		}
	}

	/**
	 * Enqueues the select script.
	 *
	 * @since 1.30.0
	 */
	public static function admin_enqueue_scripts() {
		global $pagenow;
		if ( $pagenow == 'edit.php' ) {
			$post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : null;
			if ( $post_type === 'product' ) {
				Groups_UIE::enqueue( 'select' );
			}
		}
	}

	/**
	 * Sorting is done with posts_join and posts_orderby instead.
	 *
	 * @deprecated
	 */
	public static function admin_init() {
		$screen = get_current_screen();
		if ( isset( $screen->id ) && ( $screen->id == 'edit-product' ) ) {
			add_filter( 'request', array( __CLASS__, 'request' ) );
		}
	}

	/**
	 * Modify the $query_vars to sort by meta, but this is useless because
	 * we need to sort by the content of the related group names list.
	 *
	 * Using posts_join and posts_orderby filters instead.
	 *
	 * @deprecated
	 *
	 * @param array $query_vars
	 *
	 * @return array
	 */
	public static function request( $query_vars ) {
		if ( !isset( $query_vars['orderby'] ) || ( isset( $query_vars['orderby'] ) && 'groups-wc' == $query_vars['orderby'] ) ) {
			$query_vars = array_merge( $query_vars, array(
				'meta_key' => '_groups_groups',
				'orderby' => 'meta_value'
			) );
		}
		return $query_vars;
	}

	/**
	 * Add the Groups column to the product overview screen.
	 *
	 * @param array $posts_columns
	 *
	 * @return array
	 */
	public static function manage_product_posts_columns( $posts_columns ) {
		$posts_columns['groups-wc'] = sprintf(
			'<span title="%s">%s</span>',
			esc_attr__( 'Groups to which customer is added and removed (within parenthesis).', 'groups-woocommerce' ),
			esc_html__( 'Groups', 'groups-woocommerce' )
		);
		return $posts_columns;
	}

	/**
	 * Adds sortable columns.
	 *
	 * @param array $posts_columns
	 *
	 * @return array
	 */
	public static function manage_edit_product_sortable_columns( $posts_columns ) {
		$posts_columns['groups-wc'] = 'groups-wc';
		return $posts_columns;
	}

	/**
	 * Renders the additional Groups column's content for each entry.
	 *
	 * @param string $column_name
	 * @param int $post_id
	 */
	public static function manage_product_posts_custom_column( $column_name, $post_id ) {
		switch ( $column_name ) {
			case 'groups-wc' :
				echo self::render_group_list( $post_id );
				break;
		}
	}

	/**
	 * Renders the group filter field to filter products by group added or removed.
	 *
	 * @since 1.30.0
	 */
	public static function restrict_manage_posts() {

		global $pagenow;

		if ( is_admin() ) {
			if ( $pagenow == 'edit.php' ) {
				$post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : null;
				if ( $post_type === 'product' ) {
					// Product Groups ...
					$output = '';
					$output .= '<div class="groups-woocommerce-groups-container">';
					$output .= sprintf(
						'<select class="groups-woocommerce-product-groups" name="%s[]" multiple="multiple" placeholder="%s" data-placeholder="%s">',
						esc_attr( 'groups-woocommerce-product-groups' ),
						esc_attr( __( 'Product Groups &hellip;', 'groups-woocommerce' ) ) ,
						esc_attr( __( 'Product Groups &hellip;', 'groups-woocommerce' ) )
					);

					$previous_selected = array();
					if ( !empty( $_GET['groups-woocommerce-product-groups'] ) ) {
						$previous_selected = $_GET['groups-woocommerce-product-groups'];
						if ( !is_array( $previous_selected ) ) {
							$previous_selected = array();
						}
						if ( in_array( self::ANY, $previous_selected ) ) {
							$previous_selected = array( self::ANY );
						} else if ( in_array( self::NONE, $previous_selected ) ) {
							$previous_selected = array( self::NONE );
						}
					}

					$output .= sprintf(
						'<option value="%s" %s >%s</option>', self::NONE,
						esc_attr( in_array( self::NONE, $previous_selected ) ? ' selected="selected" ' : '' ),
						esc_attr( __( '(none)', 'groups-woocommerce' ) )
					);
					$output .= sprintf(
						'<option value="%s" %s >%s</option>', self::ANY,
						esc_attr( in_array( self::ANY, $previous_selected ) ? ' selected="selected" ' : '' ),
						esc_attr( __( '(any)', 'groups-woocommerce' ) )
					);

					$groups = Groups_Group::get_groups( array( 'order_by' => 'name', 'order' => 'ASC' ) );
					foreach( $groups as $group ) {
						$selected = in_array( $group->group_id, $previous_selected ) ? ' selected="selected" ' : '';
						$output .= sprintf(
							'<option value="%s" %s >%s</option>',
							esc_attr( $group->group_id ),
							esc_attr( $selected ),
							stripslashes( wp_filter_nohtml_kses( $group->name ) )
						);
					}
					$output .= '</select>';
					$output .= '</div>';
					$output .= Groups_UIE::render_select( '.groups-woocommerce-product-groups' );

					echo $output;
				}
			}
		}
	}

	/**
	 * Render the list of groups related to a product - includes groups to which
	 * the user is added and those from which the user is removed.
	 *
	 * @param int $post_id Product's post ID
	 *
	 * @return string
	 */
	public static function render_group_list( $post_id ) {

		global $wpdb;

		wp_enqueue_style( 'groups-woocommerce-admin' );

		$output = '';

		$product_groups        = get_post_meta( $post_id, '_groups_groups', false );
		$product_groups_remove = get_post_meta( $post_id, '_groups_groups_remove', false );

		$group_table = _groups_get_tablename( 'group' );
		$groups = $wpdb->get_results( "SELECT * FROM $group_table ORDER BY name" );

		if ( count( $groups ) > 0 ) {
			$list = array();
			foreach( $groups as $group ) {
				if ( is_array( $product_groups ) && in_array( $group->group_id, $product_groups ) ) {
					$list[] = stripslashes( wp_filter_nohtml_kses( $group->name ) );
				}
				if ( is_array( $product_groups_remove ) && in_array( $group->group_id, $product_groups_remove ) ) {
					$list[] = '(' . stripslashes( wp_filter_nohtml_kses( $group->name ) ) . ')';
				}
			}
			$output .= '<div class="list-groups">';
			$output .= implode( ', ', $list );
			$output .= '</div>'; // .list-groups
		}

		// variations
		$product = wc_get_product( $post_id );
		if ( $product->is_type( 'variable' ) ) {
			$variation_ids = $product->get_children();
			foreach( $variation_ids as $variation_id ) {
				if ( $variation = wc_get_product( $variation_id ) ) {
					$variation_name = $variation->get_name();
					$variation_groups = get_post_meta( $variation_id, '_groups_variation_groups', false );
					$variation_groups_remove = get_post_meta( $variation_id, '_groups_variation_groups_remove', false );
					$list = array();
					if (
						is_array( $variation_groups ) && count( $variation_groups ) > 0 ||
						is_array( $variation_groups_remove ) && count( $variation_groups_remove ) > 0
					) {
						foreach ( $groups as $group ) {
							if ( in_array( $group->group_id, $variation_groups ) ) {
								$list[] = stripslashes( wp_filter_nohtml_kses( $group->name ) );
							}
							if ( in_array( $group->group_id, $variation_groups_remove ) ) {
								$list[] = '(' . stripslashes( wp_filter_nohtml_kses( $group->name ) ) . ')';
							}
						}
					}
					if ( count( $list ) > 0 ) {
						$output .= '<div class="variation">';
						$output .= '<span class="variation-name">';
						$output .= wp_filter_nohtml_kses( $variation_name );
						$output .= ':';
						$output .= '</span>';
						$output .= '<div class="list-groups">';
						$output .= implode( ', ', $list );
						$output .= '</div>'; // .list-groups
						$output .= '</div>'; // .variation
					}
				}
			}
		}
		return $output;
	}

	/**
	 * Filter products by group memberships added or removed via product or product variation.
	 *
	 * @since 1.30.0
	 *
	 * @param string $where
	 * @param WP_Query $query
	 *
	 * @return string
	 */
	public static function posts_where( $where, $query ) {
		global $wpdb;

		// @since 1.32.0 don't mess with unrelated queries or for orders, as we're using the same $_GET field for groups
		if ( !$query->is_main_query() ) {
			return $where;
		}

		if ( $query->query_vars['post_type'] !== 'product' ) {
			return $where;
		}

		if ( !empty( $_GET['groups-woocommerce-product-groups'] ) && is_array( $_GET['groups-woocommerce-product-groups'] ) ) {
			if ( in_array( self::ANY, $_GET['groups-woocommerce-product-groups'] ) ) {
				$condition =
					"SELECT post_id FROM $wpdb->postmeta " .
					"WHERE meta_key IN ( '_groups_groups', '_groups_groups_remove', '_groups_variation_groups', '_groups_variation_groups_remove' ) ";
				$where .=
					" AND ( $wpdb->posts.ID IN ( $condition ) OR $wpdb->posts.post_parent IN ( $condition ) ) ";
			} else if ( in_array( self::NONE, $_GET['groups-woocommerce-product-groups'] ) ) {
				$condition =
					"SELECT post_id FROM $wpdb->postmeta " .
					"WHERE meta_key IN ( '_groups_groups', '_groups_groups_remove', '_groups_variation_groups', '_groups_variation_groups_remove' ) ";
				$where .=
					" AND ( $wpdb->posts.ID NOT IN ( $condition ) AND $wpdb->posts.post_parent NOT IN ( $condition ) ) ";
			} else {
				$group_ids = array();
				foreach ( $_GET['groups-woocommerce-product-groups'] as $group_id ) {
					if ( $group_id = Groups_Utility::id( $group_id ) ) {
						if ( Groups_Group::read( $group_id ) ) {
							$group_ids[] = $group_id;
						}
					}
				}
				$group_ids = array_unique( $group_ids );
				if ( count( $group_ids ) > 0 ) {
					$groups = ' ( ' . implode( ',', esc_sql( $group_ids ) ) . ' ) ';
					$condition =
						"SELECT post_id FROM $wpdb->postmeta " .
						"WHERE meta_key IN ( '_groups_groups', '_groups_groups_remove', '_groups_variation_groups', '_groups_variation_groups_remove' ) AND meta_value IN $groups";
					$where .=
						" AND ( $wpdb->posts.ID IN ( $condition ) OR $wpdb->posts.post_parent IN ( $condition ) ) ";
				}
			}
		}
		return $where;
	}

	/**
	 * Enhance the orderby clause.
	 *
	 * @param string $orderby
	 * @param WP_Query $query
	 *
	 * @return string
	 */
	public static function posts_orderby( $orderby, $query ) {
		if ( self::use_ordering() ) {
			switch ( $query->get( 'order' ) ) {
				case 'desc' :
				case 'DESC' :
					$order = 'DESC';
					break;
				default :
					$order = 'ASC';
			}
			$prefix =
				' groups_wc.groups ' . $order .
				', groups_wc.variation_groups ' . $order .
				', groups_wc.groups_remove ' . $order .
				', groups_wc.variation_groups_remove ' . $order;
			if ( !empty( $orderby ) ) {
				$prefix .= ' , ';
			}
			$orderby = $prefix . $orderby;
		}
		return $orderby;
	}

	/**
	 * Enhance the join clause.
	 *
	 * @param string $join
	 * @param WP_Query $query
	 *
	 * @return string
	 */
	public static function posts_join( $join, $query ) {
		global $wpdb;
		if ( self::use_ordering() ) {
			$group_table = _groups_get_tablename( 'group' );
			// enhance the query to take group memberships added and removed into account
			// as well as group memberships added and removed by variations
			$join .=
				" " .
				"LEFT JOIN ( " .
					"SELECT p.ID post_id, " .
					"GROUP_CONCAT( DISTINCT _groups_groups.group_name ORDER BY _groups_groups.group_name ASC ) groups, " .
					"GROUP_CONCAT( DISTINCT _groups_groups_remove.group_name ORDER BY _groups_groups_remove.group_name ASC ) groups_remove, " .
					"GROUP_CONCAT( variation_groups.name ORDER BY variation_groups.name SEPARATOR ' ' ) variation_groups, " .
					"GROUP_CONCAT( variation_groups_remove.name ORDER BY variation_groups_remove.name SEPARATOR ' ' ) variation_groups_remove " .
					"FROM $wpdb->posts p " .
					"LEFT JOIN (" .
						"SELECT post_id, g.name group_name " .
						"FROM $wpdb->postmeta pm " .
						"LEFT JOIN $group_table g ON pm.meta_value = g.group_id " .
						"WHERE pm.meta_key = '_groups_groups' " .
					") AS _groups_groups ON p.ID = _groups_groups.post_id " .
					"LEFT JOIN (" .
						"SELECT post_id, g.name group_name " .
						"FROM $wpdb->postmeta pm " .
						"LEFT JOIN $group_table g ON pm.meta_value = g.group_id " .
						"WHERE pm.meta_key = '_groups_groups_remove' " .
					") AS _groups_groups_remove ON p.ID = _groups_groups_remove.post_id " .
					// added to groups by variations
					"LEFT JOIN (" .
						"SELECT g.name name, p.post_parent FROM $wpdb->postmeta pm " .
						"LEFT JOIN $group_table g ON pm.meta_value = g.group_id " .
						"LEFT JOIN $wpdb->posts p ON p.ID = pm.post_id " .
						"WHERE meta_key = '_groups_variation_groups' " .
					") AS variation_groups ON p.ID = variation_groups.post_parent " .
					// removed from groups by variations
					"LEFT JOIN (" .
						"SELECT g.name name, p.post_parent FROM $wpdb->postmeta pm " .
						"LEFT JOIN $group_table g ON pm.meta_value = g.group_id " .
						"LEFT JOIN $wpdb->posts p ON p.ID = pm.post_id " .
						"WHERE meta_key = '_groups_variation_groups_remove' " .
					") AS variation_groups_remove ON p.ID = variation_groups_remove.post_parent " .
					// grouping
					"GROUP BY p.ID " .
				") groups_wc ON $wpdb->posts.ID = groups_wc.post_id ";
		}
		return $join;
	}

	/**
	 * Whether to add our ordering by groups.
	 *
	 * @return boolean
	 */
	private static function use_ordering() {
		return is_admin() && !empty( $_REQUEST['orderby'] ) && $_REQUEST['orderby'] === 'groups-wc';
	}
}
Groups_WS_Admin_Product::init();
